@@ -4,6 +4,8 @@ import shlex
 import subprocess
 from collections import namedtuple
 
+from psycopg.sql import SQL
+
 from .main import special_command, RAW_QUERY
 
 TableInfo = namedtuple(
@@ -15,8 +17,8 @@ TableInfo = namedtuple(
         "hasrules",
         "hastriggers",
         "hasoids",
-        "tablespace",
         "reloptions",
+        "tablespace",
         "reloftype",
         "relpersistence",
         "relispartition",
@@ -28,38 +30,47 @@ log = logging.getLogger(__name__)
 
 @special_command("\\l", "\\l[+] [pattern]", "List databases.", aliases=("\\list",))
 def list_databases(cur, pattern, verbose):
-    query = '''SELECT d.datname as "Name",
-        pg_catalog.pg_get_userbyid(d.datdba) as "Owner",
-        pg_catalog.pg_encoding_to_char(d.encoding) as "Encoding",
-        d.datcollate as "Collate",
-        d.datctype as "Ctype",
-        pg_catalog.array_to_string(d.datacl, E'\n') AS "Access privileges"'''
+    params = {}
+    query = SQL(
+        """SELECT d.datname as name,
+        pg_catalog.pg_get_userbyid(d.datdba) as owner,
+        pg_catalog.pg_encoding_to_char(d.encoding) as encoding,
+        d.datcollate as collate,
+        d.datctype as ctype,
+        pg_catalog.array_to_string(d.datacl, E'\n') AS access_privileges
+        {verbose_fields}
+        FROM pg_catalog.pg_database d
+        {verbose_tables}
+        {pattern_where}
+        ORDER BY 1"""
+    )
     if verbose:
-        query += ''',
+        params["verbose_fields"] = SQL(
+            """,
             CASE WHEN pg_catalog.has_database_privilege(d.datname, 'CONNECT')
                     THEN pg_catalog.pg_size_pretty(pg_catalog.pg_database_size(d.datname))
                     ELSE 'No Access'
-            END as "Size",
+            END as size,
             t.spcname as "Tablespace",
-            pg_catalog.shobj_description(d.oid, 'pg_database') as "Description"'''
-    query += """
-    FROM pg_catalog.pg_database d
-    """
-    if verbose:
-        query += """
-        JOIN pg_catalog.pg_tablespace t on d.dattablespace = t.oid
-        """
-    params = {}
+            pg_catalog.shobj_description(d.oid, 'pg_database') as description"""
+        )
+        params["verbose_tables"] = SQL(
+            """JOIN pg_catalog.pg_tablespace t on d.dattablespace = t.oid"""
+        )
+    else:
+        params["verbose_fields"] = SQL("")
+        params["verbose_tables"] = SQL("")
+
     if pattern:
-        query += """
-        WHERE d.datname ~ %(schema_pattern)s
-        """
         _, schema = sql_name_pattern(pattern)
-        params["schema_pattern"] = schema
-    query = query + " ORDER BY 1"
-    cur.execute(query, params)
+        params["pattern_where"] = SQL("""WHERE d.datname ~ {}""").format(schema)
+    else:
+        params["pattern_where"] = SQL("")
+    formatted_query = query.format(**params)
+    log.debug(formatted_query.as_string(cur))
+    cur.execute(formatted_query)
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
     else:
         return [(None, None, None, cur.statusmessage)]
@@ -71,8 +82,11 @@ def list_roles(cur, pattern, verbose):
     Returns (title, rows, headers, status)
     """
 
+    params = {}
+
     if cur.connection.info.server_version > 90000:
-        sql = """
+        sql = SQL(
+            """
             SELECT r.rolname,
                 r.rolsuper,
                 r.rolinherit,
@@ -82,17 +96,22 @@ def list_roles(cur, pattern, verbose):
                 r.rolconnlimit,
                 r.rolvaliduntil,
                 ARRAY(SELECT b.rolname FROM pg_catalog.pg_auth_members m JOIN pg_catalog.pg_roles b ON (m.roleid = b.oid) WHERE m.member = r.oid) as memberof,
+                {verbose}
+                r.rolreplication
+            FROM pg_catalog.pg_roles r
+                {pattern}
+            ORDER BY 1
             """
+        )
         if verbose:
-            sql += """
-                pg_catalog.shobj_description(r.oid, 'pg_authid') AS description,
-            """
-        sql += """
-            r.rolreplication
-        FROM pg_catalog.pg_roles r
-        """
+            params["verbose"] = SQL(
+                """pg_catalog.shobj_description(r.oid, 'pg_authid') AS description, """
+            )
+        else:
+            params["verbose"] = SQL("")
     else:
-        sql = """
+        sql = SQL(
+            """
             SELECT u.usename AS rolname,
                 u.usesuper AS rolsuper,
                 true AS rolinherit,
@@ -104,46 +123,49 @@ def list_roles(cur, pattern, verbose):
                 ARRAY(SELECT g.groname FROM pg_catalog.pg_group g WHERE u.usesysid = ANY(g.grolist)) as memberof
             FROM pg_catalog.pg_user u
             """
+        )
 
-    params = {}
     if pattern:
         _, schema = sql_name_pattern(pattern)
-        sql += "WHERE r.rolname ~ %(rolname)s"
-        params["rolname"] = schema
-    sql += " ORDER BY 1"
+        params["pattern"] = SQL("WHERE r.rolname ~ {}").format(schema)
+    else:
+        params["pattern"] = SQL("")
 
-    log.debug("%s, %s", sql, params)
-    cur.execute(sql, params)
+    formatted_query = sql.format(**params)
+    log.debug(formatted_query.as_string(cur))
+    cur.execute(formatted_query)
     if cur.description:
         headers = [x.name for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
 
 
-@special_command("\\dp", "\\dp [pattern]", "List roles.", aliases=("\\z",))
+@special_command("\\dp", "\\dp [pattern]", "List privileges.", aliases=("\\z",))
 def list_privileges(cur, pattern, verbose):
     """Returns (title, rows, headers, status)"""
-    sql = """
-        SELECT n.nspname as "Schema",
-          c.relname as "Name",
+    sql = SQL(
+        """
+        SELECT n.nspname as schema,
+          c.relname as name,
           CASE c.relkind WHEN 'r' THEN 'table'
                          WHEN 'v' THEN 'view'
                          WHEN 'm' THEN 'materialized view'
                          WHEN 'S' THEN 'sequence'
                          WHEN 'f' THEN 'foreign table'
-                         WHEN 'p' THEN 'partitioned table' END as "Type",
-          pg_catalog.array_to_string(c.relacl, E'\n') AS "Access privileges",
+                         WHEN 'p' THEN 'partitioned table' END as type,
+          pg_catalog.array_to_string(c.relacl, E'\n') AS access_privileges,
+
           pg_catalog.array_to_string(ARRAY(
             SELECT attname || E':\n  ' || pg_catalog.array_to_string(attacl, E'\n  ')
             FROM pg_catalog.pg_attribute a
             WHERE attrelid = c.oid AND NOT attisdropped AND attacl IS NOT NULL
-          ), E'\n') AS "Column privileges",
+          ), E'\n') AS column_privileges,
           pg_catalog.array_to_string(ARRAY(
             SELECT polname
             || CASE WHEN NOT polpermissive THEN
                E' (RESTRICTIVE)'
                ELSE '' END
             || CASE WHEN polcmd != '*' THEN
-                   E' (' || polcmd || E'):'
+                   E' (' || polcmd::pg_catalog.text || E'):'
                ELSE E':'
                END
             || CASE WHEN polqual IS NOT NULL THEN
@@ -165,73 +187,77 @@ def list_privileges(cur, pattern, verbose):
                END
             FROM pg_catalog.pg_policy pol
             WHERE polrelid = c.oid), E'\n')
-            AS "Policies"
+            AS policies
         FROM pg_catalog.pg_class c
              LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
     """
+    )
 
-    where_clause = """
+    if pattern:
+        schema, table = sql_name_pattern(pattern)
+        if table:
+            pattern = SQL(
+                " AND c.relname OPERATOR(pg_catalog.~) {} COLLATE pg_catalog.default "
+            ).format(table)
+        if schema:
+            pattern += SQL(
+                " AND n.nspname OPERATOR(pg_catalog.~) {} COLLATE pg_catalog.default "
+            ).format(schema)
+    else:
+        pattern = SQL(" AND pg_catalog.pg_table_is_visible(c.oid) ")
+
+    where_clause = SQL(
+        """
         WHERE c.relkind IN ('r','v','m','S','f','p')
           {pattern}
           AND n.nspname !~ '^pg_'
     """
+    ).format(pattern=pattern)
 
-    params = {}
-    if pattern:
-        schema, table = sql_name_pattern(pattern)
-        if table:
-            pattern = " AND c.relname OPERATOR(pg_catalog.~) %(table_pattern)s COLLATE pg_catalog.default "
-            params["table_pattern"] = table
-        if schema:
-            pattern = (
-                pattern
-                + " AND n.nspname OPERATOR(pg_catalog.~) %(schema_pattern)s COLLATE pg_catalog.default "
-            )
-            params["schema_pattern"] = schema
-    else:
-        pattern = " AND pg_catalog.pg_table_is_visible(c.oid) "
+    sql += where_clause + SQL(" ORDER BY 1, 2 ")
 
-    where_clause = where_clause.format(pattern=pattern)
-    sql += where_clause + " ORDER BY 1, 2"
-
-    log.debug("%s, %s", sql, params)
-    cur.execute(sql, params)
+    log.debug(sql.as_string(cur))
+    cur.execute(sql)
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
 
 
 @special_command("\\ddp", "\\ddp [pattern]", "Lists default access privilege settings.")
 def list_default_privileges(cur, pattern, verbose):
     """Returns (title, rows, headers, status)"""
-    sql = """
-    SELECT pg_catalog.pg_get_userbyid(d.defaclrole) AS "Owner",
-    n.nspname AS "Schema",
+    sql = SQL(
+        """
+    SELECT pg_catalog.pg_get_userbyid(d.defaclrole) AS owner,
+    n.nspname AS schema,
     CASE d.defaclobjtype WHEN 'r' THEN 'table'
                          WHEN 'S' THEN 'sequence'
                          WHEN 'f' THEN 'function'
                          WHEN 'T' THEN 'type'
-                         WHEN 'n' THEN 'schema' END AS "Type",
-    pg_catalog.array_to_string(d.defaclacl, E'\n') AS "Access privileges"
+                         WHEN 'n' THEN 'schema' END as type,
+    pg_catalog.array_to_string(d.defaclacl, E'\n') AS access_privileges
     FROM pg_catalog.pg_default_acl d
         LEFT JOIN pg_catalog.pg_namespace n ON n.oid = d.defaclnamespace
+        {where_clause}
+    ORDER BY 1, 2, 3
     """
+    )
 
-    where_clause = """
-        WHERE (n.nspname OPERATOR(pg_catalog.~) '^({pattern})$' COLLATE pg_catalog.default
-        OR pg_catalog.pg_get_userbyid(d.defaclrole) OPERATOR(pg_catalog.~) '^({pattern})$' COLLATE pg_catalog.default)
-    """
-
+    params = {}
     if pattern:
-        _, schema = sql_name_pattern(pattern)
-        where_clause = where_clause.format(pattern=pattern)
-        sql += where_clause
+        params["where_clause"] = SQL(
+            """
+            WHERE (n.nspname OPERATOR(pg_catalog.~) {pattern} COLLATE pg_catalog.default
+            OR pg_catalog.pg_get_userbyid(d.defaclrole) OPERATOR(pg_catalog.~) {pattern} COLLATE pg_catalog.default)
+        """
+        ).format(pattern=f"^({pattern})$")
+    else:
+        params["where_clause"] = SQL("")
 
-    sql += " ORDER BY 1, 2, 3"
-    log.debug(sql)
-    cur.execute(sql)
+    log.debug(sql.format(**params).as_string(cur))
+    cur.execute(sql.format(**params))
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
 
 
@@ -241,33 +267,36 @@ def list_tablespaces(cur, pattern, **_):
     Returns (title, rows, headers, status)
     """
 
+    params = {}
     cur.execute(
         "SELECT EXISTS(SELECT * FROM pg_proc WHERE proname = 'pg_tablespace_location')"
     )
     (is_location,) = cur.fetchone()
 
-    sql = """SELECT n.spcname AS "Name",
-    pg_catalog.pg_get_userbyid(n.spcowner) AS "Owner","""
-
-    sql += (
-        " pg_catalog.pg_tablespace_location(n.oid)"
-        if is_location
-        else " 'Not supported'"
+    sql = SQL(
+        """SELECT n.spcname AS name, pg_catalog.pg_get_userbyid(n.spcowner) AS owner,
+                {location} AS location FROM pg_catalog.pg_tablespace n
+                {pattern}
+                ORDER BY 1
+              """
     )
-    sql += """ AS "Location"
-    FROM pg_catalog.pg_tablespace n"""
 
-    params = {}
+    if is_location:
+        params["location"] = SQL(" pg_catalog.pg_tablespace_location(n.oid)")
+    else:
+        params["location"] = SQL(" 'Not supported'")
+
     if pattern:
         _, tbsp = sql_name_pattern(pattern)
-        sql += " WHERE n.spcname ~ %(spcname)s"
-        params["spcname"] = tbsp
+        params["pattern"] = SQL(" WHERE n.spcname ~ {}").format(tbsp)
+    else:
+        params["pattern"] = SQL("")
 
-    sql += " ORDER BY 1"
-    log.debug("%s, %s", sql, params)
-    cur.execute(sql, params)
+    formatted_query = sql.format(**params)
+    log.debug(formatted_query.as_string(cur))
+    cur.execute(formatted_query)
 
-    headers = [x.name for x in cur.description] if cur.description else None
+    headers = [titleize(x.name) for x in cur.description] if cur.description else None
     return [(None, cur, headers, cur.statusmessage)]
 
 
@@ -277,33 +306,34 @@ def list_schemas(cur, pattern, verbose):
     Returns (title, rows, headers, status)
     """
 
-    sql = (
-        '''SELECT n.nspname AS "Name",
-    pg_catalog.pg_get_userbyid(n.nspowner) AS "Owner"'''
-        + (
-            ''',
-    pg_catalog.array_to_string(n.nspacl, E'\\n') AS "Access privileges",
-    pg_catalog.obj_description(n.oid, 'pg_namespace') AS "Description"'''
-            if verbose
-            else ""
-        )
-        + """
-    FROM pg_catalog.pg_namespace n WHERE n.nspname """
+    params = {}
+    sql = SQL(
+        """SELECT n.nspname AS name, pg_catalog.pg_get_userbyid(n.nspowner) AS owner
+                {verbose}
+              FROM pg_catalog.pg_namespace n WHERE n.nspname
+                {pattern}
+              ORDER BY 1
+              """
     )
 
-    params = {}
+    if verbose:
+        params["verbose"] = SQL(
+            """, pg_catalog.array_to_string(n.nspacl, E'\\n') AS access_privileges, pg_catalog.obj_description(n.oid, 'pg_namespace') AS description"""
+        )
+    else:
+        params["verbose"] = SQL("")
+
     if pattern:
         _, schema = sql_name_pattern(pattern)
-        sql += "~ %(schema_name)s"
-        params["schema_name"] = schema
+        params["pattern"] = SQL("~ {}").format(schema)
     else:
-        sql += "!~ '^pg_' AND n.nspname <> 'information_schema'"
-    sql += " ORDER BY 1"
+        params["pattern"] = SQL("!~ '^pg_' AND n.nspname <> 'information_schema'")
 
-    log.debug("%s, %s", sql, params)
-    cur.execute(sql, params)
+    formatted_query = sql.format(**params)
+    log.debug(formatted_query.as_string(cur))
+    cur.execute(formatted_query)
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
 
 
@@ -311,35 +341,42 @@ def list_schemas(cur, pattern, verbose):
 @special_command("\\dx", "\\dx[+] [pattern]", "List extensions.")
 def list_extensions(cur, pattern, verbose):
     def _find_extensions(cur, pattern):
-        sql = """
-            SELECT e.extname, e.oid FROM pg_catalog.pg_extension e
+        sql = SQL(
+            """
+            SELECT e.extname, e.oid
+            FROM pg_catalog.pg_extension e
+            {pattern}
+            ORDER BY 1, 2;
         """
+        )
 
         params = {}
         if pattern:
             _, schema = sql_name_pattern(pattern)
-            sql += "WHERE e.extname ~ %(extname)s"
-            params["extname"] = schema
+            params["pattern"] += SQL("WHERE e.extname ~ {}").format(schema)
+        else:
+            params["pattern"] = SQL("")
 
-        sql += " ORDER BY 1, 2;"
-        log.debug("%s, %s", sql, params)
-        cur.execute(sql, params)
+        formatted_query = sql.format(**params)
+        log.debug(formatted_query.as_string(cur))
+        cur.execute(formatted_query)
         return cur.fetchall()
 
     def _describe_extension(cur, oid):
-        params = {"object_id": oid}
-        sql = """
+        sql = SQL(
+            """
             SELECT  pg_catalog.pg_describe_object(classid, objid, 0)
-                    AS "Object Description"
+                    AS object_description
             FROM    pg_catalog.pg_depend
             WHERE   refclassid = 'pg_catalog.pg_extension'::pg_catalog.regclass
-                    AND refobjid = %(object_id)s
+                    AND refobjid = {}
                     AND deptype = 'e'
             ORDER BY 1"""
-        log.debug("%s, %s", sql, params)
-        cur.execute(sql, params)
+        ).format(oid)
+        log.debug(sql.as_string(cur))
+        cur.execute(sql)
 
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return cur, headers, cur.statusmessage
 
     if cur.connection.info.server_version < 90100:
@@ -360,30 +397,35 @@ def list_extensions(cur, pattern, verbose):
             yield None, None, None, f"""Did not find any extension named "{pattern}"."""
         return
 
-    sql = """
-      SELECT e.extname AS "Name",
-             e.extversion AS "Version",
-             n.nspname AS "Schema",
-             c.description AS "Description"
+    sql = SQL(
+        """
+      SELECT e.extname AS name,
+             e.extversion AS version,
+             n.nspname AS schema,
+             c.description AS description
       FROM pg_catalog.pg_extension e
            LEFT JOIN pg_catalog.pg_namespace n
              ON n.oid = e.extnamespace
            LEFT JOIN pg_catalog.pg_description c
              ON c.objoid = e.oid
                 AND c.classoid = 'pg_catalog.pg_extension'::pg_catalog.regclass
+        {where_clause}
+       ORDER BY 1, 2
       """
+    )
 
     params = {}
     if pattern:
         _, schema = sql_name_pattern(pattern)
-        sql += "WHERE e.extname ~ %(extname)s"
-        params["extname"] = schema
+        params["where_clause"] = SQL("WHERE e.extname ~ {}").format(schema)
+    else:
+        params["where_clause"] = SQL("")
 
-    sql += " ORDER BY 1, 2"
-    log.debug("%s, %s", sql, params)
-    cur.execute(sql, params)
+    formatted_query = sql.format(**params)
+    log.debug(formatted_query.as_string(cur))
+    cur.execute(formatted_query)
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         yield None, cur, headers, cur.statusmessage
 
 
@@ -399,56 +441,60 @@ def list_objects(cur, pattern, verbose, relkinds):
     """
     schema_pattern, table_pattern = sql_name_pattern(pattern)
 
+    params = {"relkind": relkinds}
     if verbose:
-        verbose_columns = """
-            ,pg_catalog.pg_size_pretty(pg_catalog.pg_table_size(c.oid)) as "Size",
-            pg_catalog.obj_description(c.oid, 'pg_class') as "Description" """
+        params["verbose_columns"] = SQL(
+            """
+            ,pg_catalog.pg_size_pretty(pg_catalog.pg_table_size(c.oid)) as size,
+            pg_catalog.obj_description(c.oid, 'pg_class') as description """
+        )
     else:
-        verbose_columns = ""
+        params["verbose_columns"] = SQL("")
 
-    sql = (
-        """SELECT n.nspname as "Schema",
-                    c.relname as "Name",
+    sql = SQL(
+        """SELECT n.nspname as schema,
+                    c.relname as name,
                     CASE c.relkind
                       WHEN 'r' THEN 'table' WHEN 'v' THEN 'view'
                       WHEN 'p' THEN 'partitioned table'
                       WHEN 'm' THEN 'materialized view' WHEN 'i' THEN 'index'
                       WHEN 'S' THEN 'sequence' WHEN 's' THEN 'special'
                       WHEN 'f' THEN 'foreign table' END
-                    as "Type",
-                    pg_catalog.pg_get_userbyid(c.relowner) as "Owner"
-          """
-        + verbose_columns
-        + """
+                    as type,
+                    pg_catalog.pg_get_userbyid(c.relowner) as owner
+                    {verbose_columns}
             FROM    pg_catalog.pg_class c
                     LEFT JOIN pg_catalog.pg_namespace n
                       ON n.oid = c.relnamespace
-            WHERE   c.relkind = ANY(%(relkind)s) """
+            WHERE   c.relkind = ANY({relkind})
+                {schema_pattern}
+                {table_pattern}
+            ORDER BY 1, 2
+        """
     )
 
-    params = {"relkind": relkinds}
-
     if schema_pattern:
-        sql += " AND n.nspname ~ %(nspname)s"
-        params["nspname"] = schema_pattern
+        params["schema_pattern"] = SQL(" AND n.nspname ~ {}").format(schema_pattern)
     else:
-        sql += """
+        params["schema_pattern"] = SQL(
+            """
             AND n.nspname <> 'pg_catalog'
             AND n.nspname <> 'information_schema'
             AND n.nspname !~ '^pg_toast'
             AND pg_catalog.pg_table_is_visible(c.oid) """
+        )
 
     if table_pattern:
-        sql += " AND c.relname ~ %(relname)s"
-        params["relname"] = table_pattern
+        params["table_pattern"] = SQL(" AND c.relname ~ {}").format(table_pattern)
+    else:
+        params["table_pattern"] = SQL("")
 
-    sql += " ORDER BY 1, 2"
-
-    log.debug("%s, %s", sql, params)
-    cur.execute(sql, params)
+    formatted_query = sql.format(**params)
+    log.debug(formatted_query.as_string(cur))
+    cur.execute(formatted_query)
 
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
 
 
@@ -479,7 +525,6 @@ def list_indexes(cur, pattern, verbose):
 
 @special_command("\\df", "\\df[+] [pattern]", "List functions.")
 def list_functions(cur, pattern, verbose):
-
     if verbose:
         verbose_columns = """
             ,CASE
@@ -487,10 +532,10 @@ def list_functions(cur, pattern, verbose):
                  WHEN p.provolatile = 's' THEN 'stable'
                  WHEN p.provolatile = 'v' THEN 'volatile'
             END as "Volatility",
-            pg_catalog.pg_get_userbyid(p.proowner) as "Owner",
+            pg_catalog.pg_get_userbyid(p.proowner) as owner,
           l.lanname as "Language",
           p.prosrc as "Source code",
-          pg_catalog.obj_description(p.oid, 'pg_proc') as "Description" """
+          pg_catalog.obj_description(p.oid, 'pg_proc') as description """
 
         verbose_table = """ LEFT JOIN pg_catalog.pg_language l
                                 ON l.oid = p.prolang"""
@@ -500,8 +545,8 @@ def list_functions(cur, pattern, verbose):
     if cur.connection.info.server_version >= 110000:
         sql = (
             """
-            SELECT  n.nspname as "Schema",
-                    p.proname as "Name",
+            SELECT  n.nspname as schema,
+                    p.proname as name,
                     pg_catalog.pg_get_function_result(p.oid)
                       as "Result data type",
                     pg_catalog.pg_get_function_arguments(p.oid)
@@ -512,7 +557,7 @@ def list_functions(cur, pattern, verbose):
                         WHEN p.prorettype = 'pg_catalog.trigger'::pg_catalog.regtype
                             THEN 'trigger'
                         ELSE 'normal'
-                    END as "Type" """
+                    END as type """
             + verbose_columns
             + """
             FROM    pg_catalog.pg_proc p
@@ -525,8 +570,8 @@ def list_functions(cur, pattern, verbose):
     elif cur.connection.info.server_version > 90000:
         sql = (
             """
-            SELECT  n.nspname as "Schema",
-                    p.proname as "Name",
+            SELECT  n.nspname as schema,
+                    p.proname as name,
                     pg_catalog.pg_get_function_result(p.oid)
                       as "Result data type",
                     pg_catalog.pg_get_function_arguments(p.oid)
@@ -537,7 +582,7 @@ def list_functions(cur, pattern, verbose):
                         WHEN p.prorettype = 'pg_catalog.trigger'::pg_catalog.regtype
                             THEN 'trigger'
                         ELSE 'normal'
-                    END as "Type" """
+                    END as type """
             + verbose_columns
             + """
             FROM    pg_catalog.pg_proc p
@@ -550,15 +595,15 @@ def list_functions(cur, pattern, verbose):
     else:
         sql = (
             """
-            SELECT  n.nspname as "Schema",
-                    p.proname as "Name",
+            SELECT  n.nspname as schema,
+                    p.proname as name,
                     pg_catalog.format_type(p.prorettype, NULL) as "Result data type",
                     pg_catalog.oidvectortypes(p.proargtypes) as "Argument data types",
                      CASE
                         WHEN p.proisagg THEN 'agg'
                         WHEN p.prorettype = 'pg_catalog.trigger'::pg_catalog.regtype THEN 'trigger'
                         ELSE 'normal'
-                    END as "Type" """
+                    END as type """
             + verbose_columns
             + """
             FROM    pg_catalog.pg_proc p
@@ -592,38 +637,38 @@ def list_functions(cur, pattern, verbose):
     cur.execute(sql, params)
 
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
 
 
 @special_command("\\dT", "\\dT[S+] [pattern]", "List data types")
 def list_datatypes(cur, pattern, verbose):
-    sql = """SELECT n.nspname as "Schema",
-                    pg_catalog.format_type(t.oid, NULL) AS "Name", """
+    sql = """SELECT n.nspname as schema,
+                    pg_catalog.format_type(t.oid, NULL) AS name, """
 
     if verbose:
-        sql += r''' t.typname AS "Internal name",
+        sql += r""" t.typname AS internal_name,
                     CASE
                         WHEN t.typrelid != 0
                             THEN CAST('tuple' AS pg_catalog.text)
                         WHEN t.typlen < 0
                             THEN CAST('var' AS pg_catalog.text)
                         ELSE CAST(t.typlen AS pg_catalog.text)
-                    END AS "Size",
+                    END AS size,
                     pg_catalog.array_to_string(
                         ARRAY(
                               SELECT e.enumlabel
                               FROM pg_catalog.pg_enum e
                               WHERE e.enumtypid = t.oid
                               ORDER BY e.enumsortorder
-                          ), E'\n') AS "Elements",
+                          ), E'\n') AS elements,
                     pg_catalog.array_to_string(t.typacl, E'\n')
-                        AS "Access privileges",
+                        AS access_privileges,
                     pg_catalog.obj_description(t.oid, 'pg_type')
-                        AS "Description"'''
+                        AS description"""
     else:
         sql += """  pg_catalog.obj_description(t.oid, 'pg_type')
-                        as "Description" """
+                        as description """
 
     if cur.connection.info.server_version > 90000:
         sql += """  FROM    pg_catalog.pg_type t
@@ -669,16 +714,16 @@ def list_datatypes(cur, pattern, verbose):
     log.debug("%s, %s", sql, params)
     cur.execute(sql, params)
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
 
 
 @special_command("\\dD", "\\dD[+] [pattern]", "List or describe domains.")
 def list_domains(cur, pattern, verbose):
     if verbose:
-        extra_cols = r''',
-               pg_catalog.array_to_string(t.typacl, E'\n') AS "Access privileges",
-               d.description as "Description"'''
+        extra_cols = r""",
+               pg_catalog.array_to_string(t.typacl, E'\n') AS access_privileges,
+               d.description as description"""
         extra_joins = """
            LEFT JOIN pg_catalog.pg_description d ON d.classoid = t.tableoid
                                                 AND d.objoid = t.oid AND d.objsubid = 0"""
@@ -686,9 +731,9 @@ def list_domains(cur, pattern, verbose):
         extra_cols = extra_joins = ""
 
     sql = f"""\
-        SELECT n.nspname AS "Schema",
-               t.typname AS "Name",
-               pg_catalog.format_type(t.typbasetype, t.typtypmod) AS "Type",
+        SELECT n.nspname AS schema,
+               t.typname AS name,
+               pg_catalog.format_type(t.typbasetype, t.typtypmod) as type,
                pg_catalog.ltrim((COALESCE((SELECT (' collate ' || c.collname)
                                            FROM pg_catalog.pg_collation AS c,
                                                 pg_catalog.pg_type AS bt
@@ -703,11 +748,11 @@ def list_domains(cur, pattern, verbose):
                                              WHEN t.typdefault IS NOT NULL
                                                THEN(' default ' || t.typdefault)
                                              ELSE ''
-                                           END) AS "Modifier",
+                                           END) AS modifier,
                pg_catalog.array_to_string(ARRAY(
                  SELECT pg_catalog.pg_get_constraintdef(r.oid, TRUE)
                  FROM pg_catalog.pg_constraint AS r
-                 WHERE t.oid = r.contypid), ' ') AS "Check"{extra_cols}
+                 WHERE t.oid = r.contypid), ' ') AS check {extra_cols}
         FROM pg_catalog.pg_type AS t
            LEFT JOIN pg_catalog.pg_namespace AS n ON n.oid = t.typnamespace{extra_joins}
         WHERE t.typtype = 'd' """
@@ -731,7 +776,7 @@ def list_domains(cur, pattern, verbose):
     log.debug("%s, %s", sql, params)
     cur.execute(sql, params)
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
 
 
@@ -768,13 +813,13 @@ def list_text_search_configurations(cur, pattern, verbose):
             SELECT
               (SELECT t.alias
                FROM pg_catalog.ts_token_type(c.cfgparser) AS t
-               WHERE t.tokid = m.maptokentype ) AS "Token",
+               WHERE t.tokid = m.maptokentype ) AS token,
                    pg_catalog.btrim(ARRAY
                                       (SELECT mm.mapdict::pg_catalog.regdictionary
                                        FROM pg_catalog.pg_ts_config_map AS mm
                                        WHERE mm.mapcfg = m.mapcfg
                                          AND mm.maptokentype = m.maptokentype
-                                       ORDER BY mapcfg, maptokentype, mapseqno) :: pg_catalog.text, '{}') AS "Dictionaries"
+                                       ORDER BY mapcfg, maptokentype, mapseqno) :: pg_catalog.text, '{}') AS dictionaries
             FROM pg_catalog.pg_ts_config AS c,
                  pg_catalog.pg_ts_config_map AS m
             WHERE c.oid = %(oid)s
@@ -788,7 +833,7 @@ def list_text_search_configurations(cur, pattern, verbose):
         log.debug("%s, %s", sql, params)
         cur.execute(sql, params)
 
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return cur, headers, cur.statusmessage
 
     if cur.connection.info.server_version < 80300:
@@ -808,15 +853,18 @@ def list_text_search_configurations(cur, pattern, verbose):
                 cur, headers, status = _fetch_oid_details(cur, oid)
                 yield title, cur, headers, status
         else:
-            yield None, None, None, 'Did not find any results for pattern "{}".'.format(
-                pattern
+            yield (
+                None,
+                None,
+                None,
+                'Did not find any results for pattern "{}".'.format(pattern),
             )
         return
 
     sql = """
-        SELECT n.nspname AS "Schema",
-               c.cfgname AS "Name",
-               pg_catalog.obj_description(c.oid, 'pg_ts_config') AS "Description"
+        SELECT n.nspname AS schema,
+               c.cfgname AS name,
+               pg_catalog.obj_description(c.oid, 'pg_ts_config') AS description
         FROM pg_catalog.pg_ts_config c
         LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.cfgnamespace
         """
@@ -831,7 +879,7 @@ def list_text_search_configurations(cur, pattern, verbose):
     log.debug("%s, %s", sql, params)
     cur.execute(sql, params)
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         yield None, cur, headers, cur.statusmessage
 
 
@@ -937,7 +985,7 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
                     c.relhasoids,
                     {suffix},
                     c.reltablespace,
-                    0 AS reloftype,
+                    '' AS reloftype,
                     'p' AS relpersistence,
                     false as relispartition
                  FROM pg_catalog.pg_class c
@@ -953,7 +1001,7 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
                     c.relhasoids,
                     {suffix},
                     c.reltablespace,
-                    0 AS reloftype,
+                    '' AS reloftype,
                     'p' AS relpersistence,
                     false as relispartition
                  FROM pg_catalog.pg_class c
@@ -972,7 +1020,7 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
     # If it's a seq, fetch it's value and store it for later.
     if tableinfo.relkind == "S":
         # Do stuff here.
-        sql = f"""SELECT * FROM "{schema_name}"."{relation_name}"""
+        sql = f'''SELECT * FROM "{schema_name}"."{relation_name}"'''
         log.debug(sql)
         cur.execute(sql)
         if not (cur.rowcount > 0):
@@ -1021,8 +1069,8 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
     if tableinfo.relkind == "i" or tableinfo.relkind == "I":
         if cur.connection.info.server_version >= 110000:
             sql += (
-                f",\n CASE WHEN a.attnum <= (SELECT i.indnkeyatts FROM pg_catalog.pg_index i "
-                "WHERE i.indexrelid = '{oid}') THEN 'yes' ELSE 'no' END AS is_key"
+                ",\n CASE WHEN a.attnum <= (SELECT i.indnkeyatts FROM pg_catalog.pg_index i "
+                f"WHERE i.indexrelid = '{oid}') THEN 'yes' ELSE 'no' END AS is_key"
             )
             att_cols["indexkey"] = cols
             cols += 1
@@ -1126,7 +1174,7 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
         log.debug(sql)
         cur.execute(sql)
         if cur.rowcount > 0:
-            view_def = cur.fetchone()
+            (view_def,) = cur.fetchone()
 
     # Prepare the cells of the table to print.
     cells = []
@@ -1394,8 +1442,7 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
             if cur.rowcount > 0:
                 status.append("Indexes:\n")
             for row in cur:
-
-                # /* untranslated index name */
+                # /* untranslated indextname */
                 status.append(f'''    "{row[0]}"''')
 
                 # /* If exclusion constraint, print the constraintdef */
@@ -1873,7 +1920,7 @@ def show_function_definition(cur, pattern, verbose):
     log.debug("%s, %s", sql, params)
     cur.execute(sql, params)
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         if verbose:
             (source,) = cur.fetchone()
             rows = _FakeCursor()
@@ -1901,25 +1948,13 @@ def shell_command(cur, pattern, verbose):
 
 @special_command("\\dE", "\\dE[+] [pattern]", "List foreign tables.", aliases=())
 def list_foreign_tables(cur, pattern, verbose):
-
-    if verbose:
-        verbose_cols = """
-            , pg_catalog.pg_size_pretty(pg_catalog.pg_table_size(c.oid)) as "Size",
-            pg_catalog.obj_description(c.oid, 'pg_class') as "Description" """
-    else:
-        verbose_cols = ""
-
-    if pattern:
-        _, tbl_name = sql_name_pattern(pattern)
-        filter = f" AND c.relname OPERATOR(pg_catalog.~) '^({tbl_name})$' "
-    else:
-        filter = ""
-
-    query = f"""
-        SELECT n.nspname as "Schema",
-        c.relname as "Name",
-        CASE c.relkind WHEN 'r' THEN 'table' WHEN 'v' THEN 'view' WHEN 'm' THEN 'materialized view' WHEN 'i' THEN 'index' WHEN 'S' THEN 'sequence' WHEN 's' THEN 'special' WHEN 'f' THEN 'foreign table' WHEN 'p' THEN 'table' WHEN 'I' THEN 'index' END as "Type",
-        pg_catalog.pg_get_userbyid(c.relowner) as "Owner"
+    params = {}
+    query = SQL(
+        """
+        SELECT n.nspname as schema,
+        c.relname as name,
+        CASE c.relkind WHEN 'r' THEN 'table' WHEN 'v' THEN 'view' WHEN 'm' THEN 'materialized view' WHEN 'i' THEN 'index' WHEN 'S' THEN 'sequence' WHEN 's' THEN 'special' WHEN 'f' THEN 'foreign table' WHEN 'p' THEN 'table' WHEN 'I' THEN 'index' END as type,
+        pg_catalog.pg_get_userbyid(c.relowner) as owner
         {verbose_cols}
         FROM pg_catalog.pg_class c
             LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
@@ -1931,10 +1966,34 @@ def list_foreign_tables(cur, pattern, verbose):
         {filter}
         ORDER BY 1,2;
         """
+    )
 
-    cur.execute(query)
+    if verbose:
+        params["verbose_cols"] = SQL(
+            """
+            , pg_catalog.pg_size_pretty(pg_catalog.pg_table_size(c.oid)) as size,
+            pg_catalog.obj_description(c.oid, 'pg_class') as description """
+        )
+    else:
+        params["verbose_cols"] = SQL("")
+
+    if pattern:
+        _, tbl_name = sql_name_pattern(pattern)
+        params["filter"] = SQL(" AND c.relname OPERATOR(pg_catalog.~) {} ").format(
+            f"^({tbl_name})$"
+        )
+    else:
+        params["filter"] = SQL("")
+
+    formatted_query = query.format(**params)
+    log.debug(formatted_query.as_string(cur))
+    cur.execute(formatted_query)
     if cur.description:
-        headers = [x.name for x in cur.description]
+        headers = [titleize(x.name) for x in cur.description]
         return [(None, cur, headers, cur.statusmessage)]
     else:
         return [(None, None, None, cur.statusmessage)]
+
+
+def titleize(column):
+    return column[0].capitalize() + " ".join(c for c in column[1:].split("_"))
