@@ -122,7 +122,7 @@ def read_from_file(path):
 
 
 def _index_of_file_name(tokenlist):
-    for (idx, token) in reversed(list(enumerate(tokenlist[:-2]))):
+    for idx, token in reversed(list(enumerate(tokenlist[:-2]))):
         if token.is_keyword and token.value.upper() in ("TO", "FROM"):
             return idx + 2
     raise Exception("Missing keyword in \\copy command. Either TO or FROM is required.")
@@ -132,6 +132,7 @@ def _index_of_file_name(tokenlist):
     "\\copy",
     "\\copy [tablename] to/from [filename]",
     "Copy data between a file and a table.",
+    case_sensitive=False,
 )
 def copy(cur, pattern, verbose):
     """Copies table data to/from files"""
